#include "runtime/mem_arena.h"
#include "runtime/resources.h"
#include "runtime/threading.h"

#include "gfx/gfx.h"
#include "gfx/renderer_api.h"

#include "asset_compiler/asset_compiler.h"

void RegisterCVars(void) {
    rtRegisterAssetCompilerCVars();
}

static rt_render_graph *_graph;

static rt_result ForwardPassExecute(rt_command_buffer_handle cmdbuf,
                                    const rt_render_list *lists,
                                    uint32_t list_count,
                                    void *userdata) {
    return RT_SUCCESS;
}

/* Called after the runtime has finished its initialization and before entering the main-loop*/
void Init(void) {
    rtLog("GAME", "Init");
    rtInitAssetCompiler();

    rtWaitForAssetProcessing();

    rt_render_graph_builder builder = g_renderer.CreateRenderGraphBuilder();
    rt_attachment_info backbuffer   = {
          .name    = "backbuffer",
          .format  = RT_PIXEL_FORMAT_SWAPCHAIN,
          .width   = RT_RENDER_TARGET_SIZE_SWAPCHAIN,
          .height  = RT_RENDER_TARGET_SIZE_SWAPCHAIN,
          .samples = 1,
          .layers  = 1,
    };
    builder.AddRenderTarget(builder.obj, &backbuffer);

    rt_pass_info forward = {.name  = "forward",
                            .flags = RT_PASS_FLAG_EXECUTE_ALWAYS | RT_PASS_FLAG_GRAPHICS};
    builder.AddRenderPass(builder.obj, &forward);
    builder.AddColorOutput(builder.obj,
                           "forward",
                           "backbuffer",
                           RT_PASS_LOAD_MODE_CLEAR,
                           RT_PASS_WRITE_MODE_STORE,
                           (rt_color){.r = 1.f, .g = 0.f, .b = 1.f, .a = 1.f});
    builder.SetBackbuffer(builder.obj, "backbuffer");
    builder.BindRenderPass(builder.obj, "forward", ForwardPassExecute, NULL);
    builder.Build(builder.obj, &_graph);

    g_renderer.DestroyRenderGraphBuilder(&builder);
}

/* Called after exiting the main-loop and before the runtime starts its shutdown */
void Shutdown(void) {
    rtLog("GAME", "Shutdown");
    rtShutdownAssetCompiler();
}

void Update(unsigned int frame_id) {
    RT_UNUSED(frame_id);
}

void Render(unsigned int frame_id) {
    g_renderer.ExecuteRenderGraph(_graph);
}