#ifndef RT_RENDERER_COMMON_RENDER_GRAPH_H
#define RT_RENDERER_COMMON_RENDER_GRAPH_H

#include "gfx/renderer_api.h"
#include "runtime/mem_arena.h"

typedef struct {
    const char *name;
    rt_pixel_format format;
    unsigned int width;
    unsigned int height;

    unsigned int samples;
    unsigned int layers;

} rt_physical_render_target_info;

typedef rt_render_target_handle
rt_rgb_create_render_target_fn(const rt_physical_render_target_info *rt_info);
typedef int rt_rgb_require_explicit_synchronization_fn(void);

typedef struct {
    rt_rgb_create_render_target_fn *CreateRenderTarget;
    rt_rgb_require_explicit_synchronization_fn *RequireExplicitSynchronization;
} rt_render_graph_builder_platform_callbacks;

typedef struct {
    uint32_t flags;

    /* Used for cheap referencing */
    uint32_t id;

    /* Used for debug output */
    const char *name;

    /* Render targets */
    rt_render_target_handle *color_outputs;
    rt_color *color_clear_values;
    rt_pass_load_mode *color_loads;
    rt_pass_write_mode *color_writes;
    uint32_t color_output_count;
    rt_render_target_handle depth_stencil;
    rt_depth_stencil_value depth_stencil_clear_value;
    rt_pass_load_mode depth_stencil_load;
    rt_pass_write_mode depth_stencil_write;
    rt_render_target_handle *sampled_inputs;
    uint32_t sampled_input_count;

    /* Used for parallelisation on the CPU-side. All passes with execution level N can
     * be recorded in parallel, after passes with level N-1 have finished. */
    uint32_t execution_level;

    /* GFX layer function for executing the pass */
    rt_execute_render_pass_fn *Execute;
    void *user_data;

    /* These refer to the semaphores array */
    uint32_t first_wait;
    uint32_t wait_count;
    uint32_t first_signal;
    uint32_t signal_count;
} rt_render_pass;

struct rt_render_graph_s {
    rt_render_target_handle *render_targets;
    uint32_t render_target_count;

    rt_gpu_semaphore_handle *semaphores;
    uint32_t semaphore_count;

    rt_render_pass *passes;
    uint32_t pass_count;

    uint32_t backbuffer_index;
};

#ifdef __cplusplus
extern "C" {
#endif

rt_render_graph_builder
rtCreateRenderGraphBuilder(const rt_render_graph_builder_platform_callbacks *platform_cbs);

void rtDestroyRenderGraphBuilder(rt_render_graph_builder *builder);

#ifdef __cplusplus
}
#endif

#endif